\name{dccspec-methods}
\docType{methods}
\alias{dccspec}
\alias{dccspec,ANY-method}
\alias{dccspec,uGARCHmultispec-method}
\title{function: DCC-GARCH Specification}
\description{
Method for creating a DCC-GARCH specification object prior to fitting.
}
\usage{
dccspec(uspec, VAR = FALSE, VAR.opt = list(robust = FALSE, lag = 1, lag.max = NULL, 
lag.criterion = c("AIC", "HQ", "SC", "FPE"), external.regressors = NULL, 
robust.control = list("gamma" = 0.25, "delta" = 0.01, "nc" = 10, "ns" = 500)), 
		dccOrder = c(1,1), distribution = c("mvnorm", "mvt", "mvlaplace"),
		start.pars = list(), fixed.pars = list())
}
\arguments{
  \item{uspec}{
  A \code{\linkS4class{uGARCHmultispec}} object created by calling \sQuote{multispec} on
  a list of univariate GARCH specifications.}
  \item{VAR}{ Whether to fit a VAR model to the data.}
  \item{VAR.opt}{ The VAR model options.}
  \item{dccOrder}{ The DCC autoregressive order.}
  \item{distribution}{ The multivariate distribution. Currently the multivariate Normal, 
  Student and Laplace are implemented.}
  \item{start.pars}{ (optional) Starting values for the DCC parameters (starting values for the 
  univariate garch specification should be passed directly via the \sQuote{uspec} object).}
  \item{fixed.pars}{ (optional) Fixed DCC parameters. This is required in the \code{\link{dccfilter}}, 
  \code{\link{dccforecast}}, \code{\link{dccsim}} with spec, and  \code{\link{dccroll}} 
  methods.}
}
\value{
A \code{\linkS4class{DCCspec}} object containing details of the DCC-GARCH specification.
}
\details{
The \sQuote{VAR.opt} allows for a robust version of VAR based on the multivariate Least Trimmed Squares Estimator 
described in Croux and Joossens (2008). The \sQuote{robust.control} includes additional tuning parameters to the
robust regression including the proportion to trim (\dQuote{gamma}), the critical value for Reweighted estimator
(\dQuote{delta}), the number of subsets (\dQuote{ns}) and the number of C-steps (\dQuote{nc}).
}
\author{Alexios Ghalanos}
\examples{
# same specification for all:
uspec = multispec( replicate(4, ugarchspec()) )
# different univariate specifications
uspec = multispec( c( ugarchspec(variance.model = list(model = "sGARCH")), 
ugarchspec(variance.model = list(model = "gjrGARCH")), ugarchspec(mean.model = list(armaOrder = c(2,1))) ) )
# pass uspec into dccspec
spec = dccspec(uspec, dccOrder = c(1,1))
spec
}
\keyword{methods}