\name{goGARCHfft-class}
\docType{class}
\alias{goGARCHfft-class}
\alias{dfft,goGARCHfft-method}
\alias{dfft}
\alias{pfft,goGARCHfft-method}
\alias{pfft}
\alias{qfft,goGARCHfft-method}
\alias{qfft}
\alias{nportmoments,goGARCHfft-method}
\alias{nportmoments}
\title{Class: GO-GARCH portfolio density}
\description{Class for the GO-GARCH portfolio density}
\section{Objects from the Class}{
The class is returned by calling the function \code{\link{convolution}}  on objects of
class \code{\linkS4class{goGARCHfit}}, \code{\linkS4class{goGARCHfilter}}, \code{\linkS4class{goGARCHforecast}}
and \code{\linkS4class{goGARCHsim}}.
}
\section{Slots}{
  \describe{
    \item{\code{dist}:}{A list with the portfolio density and other details.}
  }
}
\section{Methods}{
  \describe{
    \item{dfft}{\code{signature(object = "goGARCHfft")}:
    The takes additional argument \dQuote{index} to indicate the particular time point,
    and returns an interpolated density function which may be called like any other
    \dQuote{d} type density function.
    }
    \item{pfft}{\code{signature(object = "goGARCHfft")}
    The takes additional argument \dQuote{index} to indicate the particular time point,
    and returns an interpolated distribution function which may be called like any other
    \dQuote{p} type distribution function.}
    \item{qfft}{\code{signature(object = "goGARCHfft")} 
    This takes additional argument \dQuote{index} to indicate the particular time point,
    and returns an interpolated quantile function which may be called like any other
    \dQuote{q} type quantile function. This may also be used to generate pseudo-random
    variables from the distribution by using random standard uniform numbers as inputs.}
    \item{nportmoments}{\code{signature(object = "goGARCHfft")}:
    Calculate and returns a matrix of the first 4 standardized moments by evaluation of the
    portfolio density using quadrature based method (i.e. calling R's \dQuote{integrate} 
    function on the portfolio density).}
	 }
}
\author{Alexios Ghalanos}
\examples{
\dontrun{
data(dji30ret)
spec = gogarchspec(mean.model = list(demean = "constant"), 
		variance.model = list(model = "sGARCH", garchOrder = c(1,1), 
		submodel = NULL), distribution.model = list(distribution = "manig"),
		ica = "fastica")

fit = gogarchfit(spec = spec, data  = dji30ret[,1:4, drop = FALSE],  out.sample = 50, gfun = "tanh")
forc = gogarchforecast(fit, n.ahead = 1, n.roll = 2)

portnig = convolution(forc, weights = rep(1/4, 4))

# find the forecasted 1\% and 5\% VaR at the 1-ahead forecast horizon
portq = qfft(portnig, index = 1)
portq(0.01)
portq(0.05)

# the moments:
nm = nportmoments(portnig)
print(nm, digits = 4)

# check against the geometric moments (adjustments to integrate accuracy and
# FFT parameters will lead to closer results).
gm = gportmoments(forc, weights = matrix(1/4, ncol = 4, nrow = 3))
print(gm, digits = 4)
}}
\keyword{classes}
