\name{goGARCHfilter-class}
\docType{class}
\alias{goGARCHfilter-class}
\alias{as.data.frame,goGARCHfilter-method}
\alias{coef,goGARCHfilter-method}
\alias{likelihood,goGARCHfilter-method}
\alias{fitted,goGARCHfilter-method}
\alias{residuals,goGARCHfilter-method}
\alias{convolution,goGARCHfilter-method}
\alias{gportmoments,goGARCHfilter-method}
\alias{rcokurt,goGARCHfilter-method}
\alias{rcor,goGARCHfilter-method}
\alias{rcoskew,goGARCHfilter-method}
\alias{rcov,goGARCHfilter-method}
\alias{show,goGARCHfilter-method}
\title{class: GO-GARCH Filter Class}
\description{Class for the GO-GARCH filter.}
\section{Objects from the Class}{
The class is returned by calling the function \code{\link{gogarchfilter}} and is mainly
called by \code{\link{gogarchfit}} when the \dQuote{out.sample} option is used.
}
\section{Slots}{
  \describe{
    \item{\code{mfilter}:}{Multivariate filter object.}
    \item{\code{ufilter}:}{Univariate multifilter of class \code{"uGARCHmultifilter"}.}
  }
}
\section{Extends}{
Class \code{"\linkS4class{mGARCHfilter}"}, directly.
Class \code{"\linkS4class{GARCHfilter}"}, by class "mGARCHfilter", distance 2.
Class \code{"\linkS4class{rGARCH}"}, by class "mGARCHfilter", distance 3.
}
\section{Methods}{
  \describe{
  \item{as.data.frame}{\code{signature(x = "goGARCHfilter")}:\cr
   function:\cr
   \bold{as.data.frame(x, type = "A")}\cr
   This returns four types of data.frame relating to the estimation of the independent components in the
   GO-GARCH model. Valid choices for \dQuote{A} are \dQuote{A} for the mixing matrix, \dQuote{AA} for the
   unconditional covariance matrix, \dQuote{U} for the rotational matrix and \dQuote{K} for the whitening
   matrix. Note that for the latter, usually derived by an eigen or singular value decomposition, some 
   ICA methods do return this and hence \dQuote{U} cannot be calculated.
   }
    \item{likelihood}{\code{signature(object = "goGARCHfilter")}: The quasi log-likelihood of the model, 
    which being an independent factor model is the sum of the univariate GARCH log-likelihoods 
    plus a term for the mixing matrix.}
    \item{coef}{\code{signature(object = "goGARCHfilter")}:  extraction of garch independent factor 
    model coefficients.}
    \item{fitted}{\code{signature(object = "goGARCHfilter")}: Extracts the fitted values.}
	\item{residuals}{\code{signature(object = "goGARCHfilter")}: Extracts the mean equation residual values.}
    \item{convolution}{\code{signature(object = "goGARCHfilter")}:\cr
    function:\cr
    \bold{convolution(object, weights, fft.step = 0.01, fft.by = 0.0001, fft.support = c(-1, 1), 
		support.method = c("user", "adaptive"), use.ff = TRUE, parallel = FALSE, 
		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), trace = 0,...)}\cr
    The convolution method takes a goGARCHfilter object and a weights vector or matrix and calculates the weighted
    density. If a vector is given, it must be the same length as the number of assets, otherwise a matrix with
    row dimension equal to the row dimension of the filtered dataset (i.e. less any lags).
    In the case of the multivariate normal distribution, this simply returns the linear and quadratic transformation 
    of the mean and covariance matrix, while in the multivariate affine NIG distribution this is based on the 
    numerical inversion by FFT of the characteristic function. In that case, the \dQuote{fft.step} option determines
    the stepsize for tuning the characteristic function inversion, \dQuote{fft.by} determines the resolution for 
    the equally spaced support given by \dQuote{fft.support}, while the use of the \dQuote{ff} package is recommended 
    to avoid memory problems on some systems and is turned on via the \dQuote{use.ff} option. The \dQuote{support.method} 
    option allows either a fixed support range to be given (option \sQuote{user}), else an adaptive method is used based on the 
    min and max of the assets at each point in time at the 0.00001 and 1-0.00001 quantiles. The range is equally spaced subject 
    to the \dQuote{fft.by} value but the returned object no longer makes of the \dQuote{ff} package returning instead a list. 
    Finally, the option for parallel computation is available, though it is far more efficient to use (in unix based systems only) 
    using the \dQuote{multicore} package than \dQuote{snowfall}.}
    \item{portmoments}{\code{signature(object = "goGARCHfilter")}:\cr
     function:\cr
    \bold{gportmoments(object, weights)}\cr
    Calculates the first 4 portfolio moments using the geometric properties of the model, given a
    matrix of asset weights with row dimension equal to the row dimension of the filtered 
    dataset (i.e. less any lags). The kurtosis is not returned for systems of dimensions greater 
    than 11 because of the memory issues related to calculations on the co-kurtosis tensor.
    }
    \item{rcokurt}{\code{signature(object = "goGARCHfilter")}:\cr
    function:\cr
    \bold{rcokurt(object, from = 1, to = 1)}\cr
    Returns the 'time-varying'  NxN^4 cokurtosis tensor in array format and is limited to a dimensions 
    of 11 assets. The \dQuote{from} and \dQuote{to} options indicate the time indices for which to 
    return the arrays. Because of memory issues, this is limited to 100 indices.}
    \item{rcoskew}{\code{signature(object = "goGARCHfilter")}
    function:\cr
    \bold{rcoskew(object, from = 1, to = 1)}\cr
    Returns the 'time-varying'  NxN^3 coskewness tensor in array format. The \dQuote{from} and 
    \dQuote{to} options indicate the time indices for which to return the arrays. Because of 
    memory issues, this is limited to 100 indices.}    
    \item{rcov}{\code{signature(object = "goGARCHfilter")}: Returns the time-varying NxN covariance matrix in 
    array format.}
    \item{rcor}{\code{signature(object = "goGARCHfilter")}: Returns the time-varying NxN correlation matrix in 
    array format. }
    \item{show}{\code{signature(object = "goGARCHfilter")}: Summary method for the filtered object. }
	}
}
\references{
de Athayde, G.M. and Flores Jr, R.G. \emph{On Certain Geometric Aspects of Portfolio Optimisation with Higher Moments},
2002, mimeo.\cr
Paolella, M.S.\emph{Intermediate Probability - A Computational Approach}, 2007, Wiley-Interscience.\cr
}
\author{Alexios Ghalanos}
\note{
The reference by Paolella (2007) contains more details on the algorithm for the characteristic function inversion
via FFT. The de Athayde and Flores (2002) paper is the basis for the geometric properties of the higher moment
tensors.
}
\keyword{classes}
