/*################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################*/

#include "garchsim.h"
using namespace Rcpp;

SEXP marmaxsim(SEXP armaindx, SEXP ar, SEXP ma, SEXP mu, SEXP x, SEXP res)
{
	try {
			Rcpp::NumericMatrix xx(x);
			Rcpp::NumericMatrix xres(res);
			Rcpp::NumericMatrix xmu(mu);

			double *indx = REAL(armaindx);
			int armap = (int) indx[0];
			int armaq = (int) indx[1];
			int m = (int) indx[2];
			int n = (int) indx[3];
			int nr = xx.nrow(), nc = xx.ncol(), i, j;
			int T = n + m;

			arma::mat Qx(xx.begin(), nr, nc, true);
			arma::mat Qres(xres.begin(), nr, nc, true);
			arma::mat Qmu(xmu.begin(), nr, nc, true);

			double *qar = REAL(ar);
			double *qma = REAL(ma);

			for(i=m; i<T; i++)
			{
				Qx.row(i) = Qmu.row(i);
				for( j=0; j<armap; j++ )
				{
					Qx.row(i) = Qx.row(i) + qar[j] * (Qx.row(i-(j+1)) - Qmu.row(i - (j+1)));
				}
				for( j=0; j<armaq; j++ )
				{
					Qx.row(i) = Qx.row(i) + qma[j] * Qres.row(i-(j+1));
				}
				Qx.row(i) = Qx.row(i) + Qres.row(i);
			}
			return Rcpp::List::create(Rcpp::Named("x") = Qx);
		} catch( std::exception &ex ) {
			forward_exception_to_r( ex );
	    } catch(...) {
			::Rf_error( "rgarch-->ugarchsim c++ exception (unknown reason)" );
	    }
	    return R_NilValue;
}

SEXP msgarchsim(SEXP garchindx, SEXP omega, SEXP alpha, SEXP beta, SEXP h, SEXP z, SEXP res, SEXP e, SEXP vxs)
{
	try {
		Rcpp::NumericMatrix xh(h);
		Rcpp::NumericMatrix xz(z);
		Rcpp::NumericMatrix xres(res);
		Rcpp::NumericMatrix xe(e);
		Rcpp::NumericMatrix xvxs(vxs);

		double *indx = REAL(garchindx);
		int garchp = (int) indx[0];
		int garchq = (int) indx[1];
		int m = (int) indx[2];
		int nr = xh.nrow(), nc = xh.ncol(), i, j;

		arma::mat Qh(xh.begin(), nr, nc, false);
		arma::mat Qz(xz.begin(), nr, nc, false);
		arma::mat Qres(xres.begin(), nr, nc, false);
		arma::mat Qe(xe.begin(), nr, nc, false);
		arma::mat Qvxs(xvxs.begin(), nr, nc, false);

		double *qomega = REAL(omega);
		double *qalpha = REAL(alpha);
		double *qbeta  = REAL(beta);

		for( i=m; i<nr; i++ )
		{
			Qh.row(i) = Qh.row(i) + qomega[0];
			Qh.row(i) = Qh.row(i) + Qvxs.row(i);
			for( j=0; j<garchp; j++ )
			{
				Qh.row(i) = Qh.row(i) + qalpha[j]*Qe.row(i-(j+1));
			}
			for( j=0; j<garchq; j++ )
			{
				Qh.row(i) = Qh.row(i) + qbeta[j]*Qh.row(i-(j+1));
			}
			Qres.row(i) = arma::pow( Qh.row(i), 0.5 ) % Qz.row(i);
			Qe.row(i) = Qres.row(i) % Qres.row(i);
		}
		return Rcpp::List::create(Rcpp::Named("h") = Qh, Rcpp::Named("res") = Qres);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    } catch(...) {
		::Rf_error( "rgarch-->ugarchsim c++ exception (unknown reason)" );
    }
    return R_NilValue;
}

SEXP mgjrgarchsim(SEXP garchindx, SEXP omega, SEXP alpha, SEXP beta, SEXP gm, SEXP h, SEXP z, SEXP res, SEXP e, SEXP nres, SEXP nindx, SEXP vxs)
{
	try {
		Rcpp::NumericMatrix xh(h);
		Rcpp::NumericMatrix xz(z);
		Rcpp::NumericMatrix xres(res);
		Rcpp::NumericMatrix xe(e);
		Rcpp::NumericMatrix xnres(nres);
		Rcpp::NumericMatrix xneg(nindx);
		Rcpp::NumericMatrix xvxs(vxs);

		double *indx = REAL(garchindx);
		int garchp = (int) indx[0];
		int garchq = (int) indx[1];
		int m = (int) indx[2];
		int nr = xh.nrow(), nc = xh.ncol(), i, j;

		arma::mat Qh(xh.begin(), nr, nc, false);
		arma::mat Qz(xz.begin(), nr, nc, false);
		arma::mat Qres(xres.begin(), nr, nc, false);
		arma::mat Qe(xe.begin(), nr, nc, false);
		arma::mat Qnres(xnres.begin(), nr, nc, false);
		arma::mat Qvxs(xvxs.begin(), nr, nc, false);
		arma::mat Qneg(xneg.begin(), nr, nc, false);

		double *qomega = REAL(omega);
		double *qalpha = REAL(alpha);
		double *qgamma = REAL(gm);
		double *qbeta  = REAL(beta);

		for( i=m; i<nr; i++ )
		{
			Qh.row(i) = Qh.row(i) + qomega[0];
			Qh.row(i) = Qh.row(i) + Qvxs.row(i);
			for( j=0; j<garchp; j++ )
			{
				Qh.row(i) = Qh.row(i) + qalpha[j]*Qe.row(i-(j+1)) + qgamma[j]*Qnres.row(i-(j+1));
			}
			for( j=0; j<garchq; j++ )
			{
				Qh.row(i) = Qh.row(i) + qbeta[j]*Qh.row(i-(j+1));
			}
			Qres.row(i) = arma::pow( Qh.row(i), 0.5 ) % Qz.row(i);
			Qe.row(i) = Qres.row(i) % Qres.row(i);
			Qnres.row(i) = Qneg.row(i) % Qe.row(i);
		}
		return Rcpp::List::create(Rcpp::Named("h") = Qh, Rcpp::Named("res") = Qres);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    } catch(...) {
		::Rf_error( "rgarch-->ugarchsim c++ exception (unknown reason)" );
    }
    return R_NilValue;
}

SEXP maparchsim(SEXP garchindx, SEXP omega, SEXP alpha, SEXP beta, SEXP gm, SEXP delta, SEXP h, SEXP z, SEXP res, SEXP vxs)
{
	try {
		Rcpp::NumericMatrix xh(h);
		Rcpp::NumericMatrix xz(z);
		Rcpp::NumericMatrix xres(res);
		Rcpp::NumericMatrix xvxs(vxs);

		double *indx = REAL(garchindx);
		int garchp = (int) indx[0];
		int garchq = (int) indx[1];
		int m = (int) indx[2];
		int nr = xh.nrow(), nc = xh.ncol(), i, j;

		arma::mat Qh(xh.begin(), nr, nc, false);
		arma::mat Qz(xz.begin(), nr, nc, false);
		arma::mat Qres(xres.begin(), nr, nc, false);
		arma::mat Qvxs(xvxs.begin(), nr, nc, false);

		double *qomega = REAL(omega);
		double *qalpha = REAL(alpha);
		double *qgamma = REAL(gm);
		double *qbeta  = REAL(beta);
		double *qdelta  = REAL(delta);

		for( i=m; i<nr; i++ )
		{
			Qh.row(i) = Qh.row(i) + qomega[0];
			Qh.row(i) = Qh.row(i) + Qvxs.row(i);
			for( j=0; j<garchp; j++ )
			{
				Qh.row(i) = Qh.row(i) + qalpha[j]*arma::pow(arma::abs(Qres.row(i-(j+1))) - qgamma[j]*Qres.row(i-(j+1)), qdelta[0]);
			}
			for( j=0; j<garchq; j++ )
			{
				Qh.row(i) = Qh.row(i) + qbeta[j]*arma::pow(Qh.row(i-(j+1)), qdelta[0]);
			}
			Qh.row(i) = arma::pow(Qh.row(i), 1/qdelta[0]);
			Qres.row(i) = Qh.row(i) % Qz.row(i);
		}
		return Rcpp::List::create(Rcpp::Named("h") = Qh, Rcpp::Named("res") = Qres);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    } catch(...) {
		::Rf_error( "rgarch-->ugarchsim c++ exception (unknown reason)" );
    }
    return R_NilValue;
}

SEXP mfgarchsim(SEXP garchindx, SEXP omega, SEXP alpha, SEXP beta, SEXP gm1, SEXP gm2, SEXP kdelta, SEXP lambda, SEXP h, SEXP z, SEXP res, SEXP vxs)
{
	try {
		Rcpp::NumericMatrix xh(h);
		Rcpp::NumericMatrix xz(z);
		Rcpp::NumericMatrix xres(res);
		Rcpp::NumericMatrix xvxs(vxs);

		double *indx = REAL(garchindx);
		int garchp = (int) indx[0];
		int garchq = (int) indx[1];
		int m = (int) indx[2];
		int nr = xh.nrow(), nc = xh.ncol(), i, j;

		arma::mat Qh(xh.begin(), nr, nc, false);
		arma::mat Qz(xz.begin(), nr, nc, false);
		arma::mat Qres(xres.begin(), nr, nc, false);
		arma::mat Qvxs(xvxs.begin(), nr, nc, false);

		double *qomega = REAL(omega);
		double *qalpha = REAL(alpha);
		double *qgamma1 = REAL(gm1);
		double *qgamma2 = REAL(gm2);
		double *qbeta  = REAL(beta);
		double *qkdelta  = REAL(kdelta);
		double *qlambda  = REAL(lambda);

		double cnst = 0.001 * 0.001;

		for( i=m; i<nr; i++ )
		{
			Qh.row(i) = Qh.row(i) + qomega[0];
			Qh.row(i) = Qh.row(i) + Qvxs.row(i);
			for( j=0; j<garchp; j++ )
			{
				Qh.row(i) = Qh.row(i) + qalpha[j] * ( arma::pow(arma::pow(cnst + arma::pow( Qz.row(i-(j+1)) - qgamma2[j],2), 0.5) - qgamma1[j] *(Qz.row(i-(j+1)) - qgamma2[j]), qkdelta[0] ) % arma::pow( Qh.row(i-(j+1)), qlambda[0]) );
			}
			for( j=0; j<garchq; j++ )
			{
				Qh.row(i) = Qh.row(i) + qbeta[j]*arma::pow(Qh.row(i-(j+1)), qlambda[0]);
			}
			Qh.row(i) = arma::pow(Qh.row(i), 1/qlambda[0]);
			Qres.row(i) = Qh.row(i) % Qz.row(i);
		}
		return Rcpp::List::create(Rcpp::Named("h") = Qh, Rcpp::Named("res") = Qres);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    } catch(...) {
		::Rf_error( "rgarch-->ugarchsim c++ exception (unknown reason)" );
    }
    return R_NilValue;
}

SEXP megarchsim(SEXP garchindx, SEXP omega, SEXP alpha, SEXP gm, SEXP meanz, SEXP beta, SEXP h, SEXP z, SEXP res, SEXP vxs)
{
	try {
		Rcpp::NumericMatrix xh(h);
		Rcpp::NumericMatrix xz(z);
		Rcpp::NumericMatrix xres(res);
		Rcpp::NumericMatrix xvxs(vxs);

		double *indx = REAL(garchindx);
		int garchp = (int) indx[0];
		int garchq = (int) indx[1];
		int m = (int) indx[2];
		int nr = xh.nrow(), nc = xh.ncol(), i, j;

		arma::mat Qh(xh.begin(), nr, nc, false);
		arma::mat Qz(xz.begin(), nr, nc, false);
		arma::mat Qres(xres.begin(), nr, nc, false);
		arma::mat Qvxs(xvxs.begin(), nr, nc, false);

		double *qomega = REAL(omega);
		double *qalpha = REAL(alpha);
		double *qgamma = REAL(gm);
		double *qmeanz = REAL(meanz);
		double *qbeta  = REAL(beta);

		for( i=m; i<nr; i++ )
		{
			Qh.row(i) = Qh.row(i) + qomega[0];
			Qh.row(i) = Qh.row(i) + Qvxs.row(i);
			for( j=0; j<garchp; j++ )
			{
				Qh.row(i) = Qh.row(i) + qalpha[j]*Qz.row(i-(j+1)) + qgamma[j]*arma::abs(Qz.row(i-(j+1)) - qmeanz[0]);
			}
			for( j=0; j<garchq; j++ )
			{
				Qh.row(i) = Qh.row(i) + qbeta[j]*arma::log(Qh.row(i-(j+1)));
			}
			Qh.row(i) = arma::exp( Qh.row(i) );
			Qres.row(i) = arma::pow( Qh.row(i), 0.5 ) % Qz.row(i);
		}
		return Rcpp::List::create(Rcpp::Named("h") = Qh, Rcpp::Named("res") = Qres);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    } catch(...) {
		::Rf_error( "rgarch-->ugarchsim c++ exception (unknown reason)" );
    }
    return R_NilValue;
}
