/*################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################*/

#include "rdcc.h"
using namespace Rcpp;

SEXP dccnormC1(SEXP Qbar, SEXP Z, SEXP dcca, SEXP dccb, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xZ(Z);
		int md = xQbar.nrow(), nr = xZ.nrow(), i, j;
		List output(3), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0;
		// empty matrices
		arma::mat Qt(md, md); 
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp2 = 0.0;
			llhtemp = 0.0;
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] * Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt/temp1;
			temp2 = arma::as_scalar( Zb.row(j) * ( arma::inv(Rt) * arma::trans(Zb.row(j))) );
			llhtemp = log(arma::det(Rt)) + temp2;
			llh+= llhtemp;
			llhout[j] = 0.5 * llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = 0.5 * llh;
		return(output);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" ); 
    	}
    	return R_NilValue;
}

SEXP dccnormC2(SEXP Qbar, SEXP H, SEXP Z, SEXP dcca, SEXP dccb, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xHbar(H);
		Rcpp::NumericMatrix xZ(Z);
		int md = xQbar.nrow(), nr = xZ.nrow(), hr = xHbar.nrow(), i, j;
		List output(4), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		arma::mat Ht(xHbar.begin(), hr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0, temp3 = 0.0;
		double lcons = md*log(2.0*PI);
		// empty matrices
		arma::mat Qt(md, md); 
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			Rtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp1.zeros();
			temp2 = 0.0;
			temp3 = 0.0;
			llhtemp = 0.0;
			Rt.zeros();
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] *Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt / temp1;
			Rtout[j] = Rt;
			temp2 = arma::as_scalar( Zb.row(j) * (arma::inv(Rt) * arma::trans(Zb.row(j)) ) );
			temp3 = 2 * log(arma::prod(Ht.row(j)));
			llhtemp = lcons + temp3 + log(arma::det(Rt)) + temp2;
			llh+= llhtemp;
			llhout[j] = -0.5 * llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = -0.5 * llh;
		output[3] = Rtout;
		return(output);
		
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" ); 
    	}
    	return R_NilValue;
}


SEXP dccstudentC1(SEXP Qbar, SEXP Z, SEXP dcca, SEXP dccb, SEXP nu, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xZ(Z);
		int md = xQbar.nrow(), nr = xZ.nrow(), i, j;
		List output(3), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		double *v = REAL(nu);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0;
		double lcons = Rf_lgammafn(0.5 * (v[0] + md)) - Rf_lgammafn(0.5 * v[0]) - 0.5 * md * log(PI*(v[0] - 2));
		// empty matrices
		arma::mat Qt(md, md);
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp2 = 0.0;
			llhtemp = 0.0;
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] * Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt/temp1;
			temp2 = arma::as_scalar( Zb.row(j) * ( arma::inv(Rt) * arma::trans(Zb.row(j))) );
			llhtemp = lcons - 0.5 * log(arma::det(Rt)) - 0.5 * (v[0] + md) * log(1.0 + (1.0/(v[0] - 2)) * temp2);
			llh+= llhtemp;
			llhout[j] = -1.0 * llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = -1.0 * llh;
		return(output);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}

SEXP dccstudentC2(SEXP Qbar, SEXP H, SEXP Z, SEXP dcca, SEXP dccb, SEXP nu, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xHbar(H);
		Rcpp::NumericMatrix xZ(Z);
		int md = xQbar.nrow(), nr = xZ.nrow(), hr = xHbar.nrow(), i, j;
		List output(4), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		arma::mat Ht(xHbar.begin(), hr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		double *v = REAL(nu);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0, temp3 = 0.0;
		double lcons = Rf_lgammafn(0.5 * (v[0] + md)) - Rf_lgammafn(0.5 * v[0]) - 0.5 * md * log(PI*(v[0] - 2));
		// empty matrices
		arma::mat Qt(md, md);
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			Rtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp1.zeros();
			temp2 = 0.0;
			temp3 = 0.0;
			llhtemp = 0.0;
			Rt.zeros();
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] *Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt / temp1;
			Rtout[j] = Rt;
			temp2 = arma::as_scalar( Zb.row(j) * (arma::inv(Rt) * arma::trans(Zb.row(j))));
			temp3 = log(arma::prod(Ht.row(j)));
			llhtemp = lcons - 0.5 * log(arma::det(Rt)) - temp3 - 0.5 * (v[0] + md) * log(1.0 + (1.0/(v[0] - 2)) * temp2);
			llh+= llhtemp;
			llhout[j] = llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = llh;
		output[3] = Rtout;
		return(output);

	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}

SEXP dcclaplaceC1(SEXP Qbar, SEXP Z, SEXP dcca, SEXP dccb, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xZ(Z);
		int md = xQbar.nrow(), nr = xZ.nrow(), i, j;
		List output(3), Qtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		double v = 0.5 * (2 - md);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0;
		double lcons = log(2.0) - 0.5 * md * log(2 * PI);
		// empty matrices
		arma::mat Qt(md, md);
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp1.zeros();
			temp2 = 0.0;
			llhtemp = 0.0;
			Rt.zeros();
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] *Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt / temp1;
			temp2 = arma::as_scalar( Zb.row(j) * (arma::inv(Rt) * arma::trans(Zb.row(j))));
			llhtemp = lcons - 0.5 * log(arma::det(Rt)) + 0.5 * v * log(0.5 * temp2) + log(Rf_bessel_k(sqrt(2 * temp2), v, 1));
			llh+= llhtemp;
			llhout[j] = -1.0 * llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = -1.0 * llh;
		return(output);

	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}


SEXP dcclaplaceC2(SEXP Qbar, SEXP H, SEXP Z, SEXP dcca, SEXP dccb, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xHbar(H);
		Rcpp::NumericMatrix xZ(Z);
		int md = xQbar.nrow(), nr = xZ.nrow(), hr = xHbar.nrow(), i, j;
		List output(4), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		arma::mat Ht(xHbar.begin(), hr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		double v = 0.5 * (2 - md);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0, temp3 = 0.0;
		double lcons = log(2.0) - 0.5 * md * log(2 * PI);
		// empty matrices
		arma::mat Qt(md, md);
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			Rtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp1.zeros();
			temp2 = 0.0;
			temp3 = 0.0;
			llhtemp = 0.0;
			Rt.zeros();
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] *Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt / temp1;
			Rtout[j] = Rt;
			temp2 = arma::as_scalar( Zb.row(j) * (arma::inv(Rt) * arma::trans(Zb.row(j))));
			temp3 = log(arma::prod(Ht.row(j)));
			llhtemp = lcons - 0.5 * log(arma::det(Rt)) - temp3 + 0.5 * v * log(0.5 * temp2) + log(Rf_bessel_k(sqrt(2 * temp2), v, 1));
			llh+= llhtemp;
			llhout[j] = llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = llh;
		output[3] = Rtout;
		return(output);

	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}

SEXP dccfilter(SEXP Qbar, SEXP Rbar, SEXP Z, SEXP dcca, SEXP dccb, SEXP dccorder, SEXP dccsum)
{
	try {
			double *timp = REAL(dccorder);
			int dccP = (int) timp[0];
			int dccQ = (int) timp[1];
			double *a = REAL(dcca);
			double *b = REAL(dccb);
			int m = (int) timp[2];
			int n = (int) timp[3];
			double *sba = REAL(dccsum);
			Rcpp::NumericMatrix xQbar(Qbar);
			Rcpp::NumericMatrix xRbar(Rbar);
			Rcpp::NumericMatrix xZ(Z);
			int md = xQbar.nrow(), nr = xZ.nrow(), i, j;
			List output(3), Qtout(n), Rtout(n), Ztout(n);
			arma::mat Qt(md, md);
			arma::mat Rt(md, md);
			arma::mat Zt(n, md);
			arma::mat Qb(xQbar.begin(), md, md, true);
			arma::mat Zb(xZ.begin(), nr, md, true);
			arma::mat Rb(xRbar.begin(), md, md, true);
			arma::mat temp1(md, md);
			arma::vec eigval(md);
			arma::mat eigvec(md, md);

			for(j=0; j<m; j++)
			{
				Qtout[j] = Qb;
				Rtout[j] = Rb;
				Zt.row(j) = Zb.row(j);
			}
			for(j=m; j<nr; j++)
			{
				Qt = Qb * (1 - sba[0]);
				for(i = 0; i<dccP; i++)
				{
					Qt = Qt + a[i]*( arma::trans( Zt.row(j-(i+1)) ) * Zt.row(j-(i+1)) );
				}
				for(i = 0; i<dccQ; i++)
				{
					arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
					Qt = Qt + b[i] *Qtemp;
				}
				Qtout[j] = Qt;
				temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
				Rt = Qt / temp1;
				Rtout[j] = Rt;
				eig_sym(eigval, eigvec, Rt);
				temp1 = ( eigvec * arma::diagmat( arma::sqrt( eigval ) ) * arma::inv( eigvec ) );
				Zt.row(j) = Zb.row(j) * temp1;
			}
			output[0] = Qtout;
			output[1] = Rtout;
			output[2] = Zt;
			return(output);
		} catch( std::exception &ex ) {
			forward_exception_to_r( ex );
	    } catch(...) {
			::Rf_error( "rgarch-->dccsim c++ exception (unknown reason)" );
	    }
	    return R_NilValue;
}

SEXP dccsim(SEXP Qbar, SEXP Rbar, SEXP Z, SEXP Res, SEXP dcca, SEXP dccb, SEXP dccorder, SEXP dccsum)
{
	try {
			double *timp = REAL(dccorder);
			int dccP = (int) timp[0];
			int dccQ = (int) timp[1];
			double *a = REAL(dcca);
			double *b = REAL(dccb);
			int m = (int) timp[2];
			int n = (int) timp[3];
			double *sba = REAL(dccsum);
			Rcpp::NumericMatrix xQbar(Qbar);
			Rcpp::NumericMatrix xRbar(Rbar);
			Rcpp::NumericMatrix xZ(Z);
			Rcpp::NumericMatrix xRes(Res);
			int md = xQbar.nrow(), nr = xZ.nrow(), i, j;
			List output(3), Qtout(n), Rtout(n), Ztout(n);
			arma::mat Qt(md, md);
			arma::mat Rt(md, md);
			arma::mat Qb(xQbar.begin(), md, md, true);
			arma::mat Zt(xZ.begin(), nr, md, true);
			arma::mat ZRes(xRes.begin(), nr, md, true);
			arma::mat Rb(xRbar.begin(), md, md, true);
			arma::mat temp1(md, md);
			arma::vec eigval(md);
			arma::mat eigvec(md, md);
			for(j=0; j<m; j++)
			{
				Qtout[j] = Qb;
				Rtout[j] = Rb;
			}
			for(j=m; j<nr; j++)
			{
				Qt = Qb * (1 - sba[0]);
				for(i = 0; i<dccP; i++)
				{
					Qt = Qt + a[i]*( arma::trans( Zt.row(j-(i+1)) ) * Zt.row(j-(i+1)) );
				}
				for(i = 0; i<dccQ; i++)
				{
					arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
					Qt = Qt + b[i] *Qtemp;
				}
				Qtout[j] = Qt;
				temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
				Rt = Qt / temp1;
				Rtout[j] = Rt;
				eig_sym(eigval, eigvec, Rt);
				temp1 = ( eigvec * arma::diagmat( arma::sqrt( eigval ) ) * arma::inv( eigvec ) );
				Zt.row(j) = ZRes.row(j) * temp1;
			}
			output[0] = Qtout;
			output[1] = Rtout;
			output[2] = Zt;
			return(output);
		} catch( std::exception &ex ) {
			forward_exception_to_r( ex );
	    } catch(...) {
			::Rf_error( "rgarch-->dccsim c++ exception (unknown reason)" );
	    }
	    return R_NilValue;
}


SEXP adccnormC1(SEXP Qbar, SEXP Z, SEXP NZ, SEXP Nbar, SEXP dcca, SEXP dccb, SEXP dccg, SEXP dccorder, SEXP dccsum, SEXP dccsumg)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xNbar(Nbar);
		Rcpp::NumericMatrix xZ(Z);
		Rcpp::NumericMatrix xNZ(NZ);
		int md = xQbar.nrow(), nr = xZ.nrow(), i, j;
		List output(3), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Nb(xNbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		arma::mat NZb(xNZ.begin(), nr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		double *g = REAL(dccg);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double *sg = REAL(dccsumg);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0;
		// empty matrices
		arma::mat Qt(md, md);
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]) - sg[0]*Nb;
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp2 = 0.0;
			llhtemp = 0.0;
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) ) + g[i]*( arma::trans( NZb.row(j-(i+1)) ) * NZb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] * Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt/temp1;
			temp2 = arma::as_scalar( Zb.row(j) * ( arma::inv(Rt) * arma::trans(Zb.row(j))) );
			llhtemp = log(arma::det(Rt)) + temp2;
			llh+= llhtemp;
			llhout[j] = 0.5 * llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = 0.5 * llh;
		return(output);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}


SEXP dccnormC2(SEXP Qbar, SEXP H, SEXP Z, SEXP NZ, SEXP Nbar, SEXP dcca, SEXP dccb, SEXP dccg, SEXP dccorder, SEXP dccsum, SEXP dccsumg)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xNbar(Nbar);
		Rcpp::NumericMatrix xNZ(NZ);
		Rcpp::NumericMatrix xHbar(H);
		Rcpp::NumericMatrix xZ(Z);
		int md = xQbar.nrow(), nr = xZ.nrow(), hr = xHbar.nrow(), i, j;
		List output(4), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), md, md, true);
		arma::mat Zb(xZ.begin(), nr, md, true);
		arma::mat Ht(xHbar.begin(), hr, md, true);
		arma::mat Nb(xNbar.begin(), md, md, true);
		arma::mat NZb(xNZ.begin(), nr, md, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		double *g = REAL(dccg);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double *sg = REAL(dccsumg);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0, temp3 = 0.0;
		double lcons = md*log(2.0*PI);
		// empty matrices
		arma::mat Qt(md, md);
		arma::mat Rt(md, md);
		arma::mat UQ(md, md);
		arma::mat temp1(md, md);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]) - sg[0]*Nb;
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			Rtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp1.zeros();
			temp2 = 0.0;
			temp3 = 0.0;
			llhtemp = 0.0;
			Rt.zeros();
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) ) + g[i]*( arma::trans( NZb.row(j-(i+1)) ) * NZb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] *Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt / temp1;
			Rtout[j] = Rt;
			temp2 = arma::as_scalar( Zb.row(j) * (arma::inv(Rt) * arma::trans(Zb.row(j)) ) );
			temp3 = 2 * log(arma::prod(Ht.row(j)));
			llhtemp = lcons + temp3 + log(arma::det(Rt)) + temp2;
			llh+= llhtemp;
			llhout[j] = -0.5 * llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = -0.5 * llh;
		output[3] = Rtout;
		return(output);

	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->dccfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}
